# -*- coding: utf-8 -*-
"""
Created on Fri May 29 15:30:55 2020

@author: Andreas Gebauer
"""

import numpy as np
import os



###############################################################################
"""
Export functions
"""

def logfile(string, logfile_path, new_logfile):
    print('\n %s' % string)
    if new_logfile == True:
        f = open(logfile_path, 'w')
        f.write('\n %s' % string)
        f.close()
    else:
        f = open(logfile_path, 'a')
        f.write('\n %s' % string)
        f.close()

def export_single_column(file, header, x):
      f = open(file, 'w')
      f.write(header)
      for i in range(len(x)):
            f.write('\n%.8E' % x[i])
      f.close()
      return

def export_two_columns(file, header, x, y):
      if len(x) != len(y):
            raise ValueError('AG_file_operations: export_two_columns: x and y have to be arrays of equal length')
      f = open(file, 'w')
      f.write(header)
      for i in range(len(x)):
            f.write('\n%14.8E %14.8E' % (x[i], y[i]))
      f.close()
      return

def export_three_columns(file, header, x, y, z):
      if len(x) != len(y) or len(x) != len(z):
            raise ValueError('AG_file_operations: export_two_columns: x and y have to be arrays of equal length')
      f = open(file, 'w')
      f.write(header)
      for i in range(len(x)):
            f.write('\n%14.8E %14.8E %14.8E' % (x[i], y[i], z[i]))
      f.close()
      return

def export_2D_array(file, header, x):
      f = open(file, 'w')
      f.write(header)
      for i in range(len(x)):
            line = '\n'
            for j in range(len(x[i])):
                  line = '%s %21.14E' % (line, x[i][j])
            f.write(line)
      f.close()
      return

def export_initial_wavefunction_for_FFT(output, z_axis, Phi0, V0, initial_energy):
    if not os.path.exists(output):
        os.makedirs(output)
    f = open(('%s/initial_wavefunction.txt' % output), 'w')
    f.write('E0 = %.5E a.u. = %.3f eV \n' % (initial_energy, initial_energy*27.2))
    f.write('# %9s %15s %15s' % ('z (a.u.)', 'Phi0(z)', 'V0(z) (a.u.)'))
    for i in range(len(z_axis)):
        f.write('\n%11.3f %15.12E %15.12E' % (z_axis[i], Phi0[i], V0[i]))
    f.close()

def export_wavefunction_time_and_space(output, name, header, x_axis, t_axis, Psi):
    if not os.path.exists(output):
        os.makedirs(output)
    f = open(('%s/%s.txt' % (output, name)), 'w')
    f.write(header)
    line = '\nt: %8s' % (' ')
    for i in range(len(t_axis)):
          line = '%s %13.6E' % (line, t_axis[i])
    f.write(line)
    for j in range(len(x_axis)):
          line = '\n%11.3f' % x_axis[j]
          for i in range(len(t_axis)): 
                line = '%s %13.6E' % (line, Psi[i][j])
          f.write(line)
    f.close()

def export_tof_spectrogram(file, header, tof_axis, t_axis, counts):
    f = open(file, 'w')
    f.write(header)
    line = '\nt: %3s' % (' ')
    for i in range(len(t_axis)):
        line = '%s %7.4f' % (line, t_axis[i])
    f.write(line)
    for j in range(len(tof_axis)):
        line = '\n%6.2f' % tof_axis[j]
        for i in range(len(t_axis)):
            line = '%s %7i' % (line, counts[i][j])
        f.write(line)
    f.close()
    
def export_Ekin_spectrogram(file, header, Ekin_axis, t_axis, counts):
    f = open(file, 'w')
    f.write(header)
    line = '\nt: %4s' % (' ')
    for i in range(len(t_axis)):
        line = '%s %10.4f' % (line, t_axis[i])
    f.write(line)
    for j in range(len(Ekin_axis)):
        line = '\n%7.3f' % Ekin_axis[j]
        for i in range(len(t_axis)):
            line = '%s %10.3f' % (line, counts[i][j])
        f.write(line)
    f.close()
    


###############################################################################
"""
Read functions
"""

def read_single_column(file):
      f = open(file)
      lines = f.readlines()
      f.close()
      x = []
      for line in lines:
            if line.startswith('#'):
                  continue
            x.append(float(line))
      return np.array(x)

def read_two_columns(file):
      f = open(file)
      lines = f.readlines()
      f.close()
      x = []
      y = []
      for line in lines:
            if line.startswith('#'):
                  continue
            s = line.split()
            x.append(float(s[0]))
            y.append(float(s[1]))
      return np.array(x), np.array(y)

def read_three_columns(file):
      f = open(file)
      lines = f.readlines()
      f.close()
      x = []
      y = []
      z = []
      for line in lines:
            if line.startswith('#'):
                  continue
            s = line.split()
            x.append(float(s[0]))
            y.append(float(s[1]))
            z.append(float(s[2]))
      return np.array(x), np.array(y), np.array(z)

def read_2D_array(file):
      f = open(file)
      lines = f.readlines()
      f.close()
      x = []
      for line in lines:
            if line.startswith('#'):
                  continue
            x_i = []
            s = line.split()
            for i in range(len(s)):
                  x_i.append(float(s[i]))
            x.append(x_i)
      return np.array(x)

###############################################################################
"""
Export functions for special cases
"""

def export_2_columns_integer_string(file, header, x, s):
    if len(x) != len(s):
        raise ValueError('AG_file_operations: export_two_columns: x and s have to be arrays of equal length')
    f = open(file, 'w')
    f.write(header)
    for i in range(len(x)):
        f.write('\n%5i %s' % (x[i], s[i]))
    f.close()
    return

###############################################################################
"""
Read functions for special cases
"""

def load_initial_wavefunction(file):
    data = open(file)
    lines = data.readlines()
    data.close()
    z_axis = []
    Phi0 = []
    V0 = []
    print(lines[0])
    print(lines[1])
    for line in lines:
        if line.startswith('E0 =') or line.startswith('E_0'):
            print(line)
            s = line.split()
            print(s[2])
            initial_energy = float(s[2])
            continue
        elif line.startswith('#'):
            continue
        s = line.split()
        z_axis.append(float(s[0]))
        Phi0.append(float(s[1]))
        V0.append(float(s[2]))
    return initial_energy, np.array(z_axis), np.array(Phi0), np.array(V0)

def load_EUV_file(EUV_timedomain_file):
    data = open(EUV_timedomain_file)
    lines = data.readlines()
    data.close()
    t_axis_EUV = []
    E_EUV = []
    E0_EUV = []
    t_EUV_min = -10
    t_EUV_max = 10
    for line in lines:
        if line.startswith('tmin='):
            s = line.split()
            t_EUV_min = float(s[-1])
            continue
        elif line.startswith('tmax='):
            s = line.split()
            t_EUV_max = float(s[-1])
            continue
        elif line.startswith('w0='):
            s = line.split()
            w0 = float(s[-1])
            continue
        elif line.startswith('#'):
            continue
        s = line.split()
        t_axis_EUV.append(float(s[0]))
        E0_EUV.append(float(s[1]))
        E_EUV.append(float(s[2]))
    return np.array(t_axis_EUV), np.array(E0_EUV), np.array(E_EUV), t_EUV_min, t_EUV_max, w0

def load_IR_field_distribution(IR_potential_filepath):
    data = open(IR_potential_filepath)
    lines = data.readlines()
    data.close()
    z_IR = []
    E_IR_Re = []
    E_IR_Im = []
    for line in lines:
        if line.startswith('#'):
            continue
        s = line.split()
        z_IR.append(float(s[0]))
        E_IR_Re.append(float(s[1]))
        E_IR_Im.append(float(s[2]))
    return np.array(z_IR), np.array(E_IR_Re), np.array(E_IR_Im)

def read_wavefunction_time_and_space(file):
      data = open(file)
      lines = data.readlines()
      data.close()
      t = []
      x = []
      Psi = []
      for line in lines:
            if line.startswith('#'):
                  continue
            elif line.startswith('t:'):
                  s = line.split()
                  for i in range(1, len(s)):
                        t.append(float(s[i]))
            else:
                  s = line.split()
                  x.append(float(s[0]))
                  Psi_x = []
                  for i in range(1, len(s)):
                        Psi_x.append(float(s[i]))
                  Psi.append(Psi_x)
      return np.array(t), np.array(x), np.transpose(np.array(Psi))

def read_2_columns_integer_string(file):
    data = open(file)
    lines = data.readlines()
    data.close()
    n = []
    string = []
    for line in lines:
        if line.startswith('#'):
            continue
        s = line.split()
        n.append(float(s[0]))
        string.append(s[1])
    return n, string

def read_SPIDER_file(file):
      data = open(file)
      lines = data.readlines()
      data.close()
      
      lambda_SPIDER = []
      I_SPIDER = []
      f = []
      I_fundamental_reconstructed = []
      phase = []
      GD = []
      GDD = []
      lambda_fundamental = []
      I_fundamental = []
      t = []
      I_t = []
      for line in lines:
            if line.startswith('#'):
                  continue
            s = line.split()
            if s[0] != '?':
                  lambda_SPIDER.append(float(s[0]))
            if s[1] != '?':
                  I_SPIDER.append(float(s[1]))
            if s[2] != '?':
                  f.append(float(s[2]))
            if s[3] != '?':
                  I_fundamental_reconstructed.append(float(s[3]))
            if s[4] != '?':
                  phase.append(float(s[4]))
            if s[5] != '?':
                  GD.append(float(s[5]))
            if s[6] != '?':
                  GDD.append(float(s[6]))
            if s[7] != '?':
                  lambda_fundamental.append(float(s[7]))
            if s[8] != '?':
                  I_fundamental.append(float(s[8]))
            if s[9] != '?':
                  t.append(float(s[9]))
            if s[10] != '?':
                  I_t.append(float(s[10]))
      return np.array(lambda_SPIDER), np.array(I_SPIDER), np.array(f), np.array(I_fundamental_reconstructed), np.array(phase), np.array(GD), np.array(GDD), np.array(lambda_fundamental), np.array(I_fundamental), np.array(t), np.array(I_t)




###############################################################################

def reshape_2D_array_for_scatterplot(X_mesh, Y_mesh, data_mesh):
      shape = data_mesh.shape
      data_array = data_mesh.reshape(shape[0] * shape[1])
      X_array = X_mesh.reshape(shape[0] * shape[1])
      Y_array = Y_mesh.reshape(shape[0] * shape[1])
      return X_array, Y_array, data_array
      
      