# -*- coding: utf-8 -*-
"""
Created on Thu Jul 25 15:50:35 2024

@author: agebauer
"""
import AG_THzTDS_m10_website as TDS
import numpy as np
import os
import sys


sys.path.append(os.path.dirname(os.path.abspath(__file__)))



### Test Data ###


script_dir = os.path.dirname(os.path.abspath(__file__))  # Current script location
parent_dir = os.path.dirname(script_dir)  # Move up one level
test_data_dir = os.path.join(parent_dir, "Test Data")  # Navigate into "Test Data"

number_measurements = 8

for i in range(number_measurements):
    SampleFirst = {'name': 'MgO_RT_%i' % (i),
          'file_reference': os.path.join(test_data_dir, f'File2000avg_22072024_s0_Air_L{i}.csv'),
          'file_sample': os.path.join(test_data_dir, f'File2000avg_22072024_s1_MgO_L{i}.csv'),
          'file_background': None,
          'outputpath': os.path.join(test_data_dir, 'MgO_TestData_SolverV3'),
          'subtract_background': False,
          'd_estimate': 0.50e-3,
          'nmin': 2.4,
          'nmax': 4.5,
          'n_estimate': 3.15,
          'kmin': 1e-7,
          'kmax': 1e-1,
          'T_window': 8.0,
          'window_t_offset': 1.0,
          'window_type': 'supergauss',#'Kaiser-Bessel',#'rect'
          'window_parameter': 8,
          'fmin': 0.3,
          'fmax': 4.0,
          'model': 'single_layer',
          'model_parameters': None,
          'interpolate': True,
          'interpolation_method': 'cubic',
          'interpolation_dt_ratio': 1e-3,
          'thickness_averaging': 'fixed_interval', #'fixed_interval', #'weighted', #'threshold'
          'average_fmin': 0.4,
          'average_fmax': 2.5,
          'average_threshold': 0.1}
    print(SampleFirst['name'])
    
    Sample_res1 = TDS.evaluate_Teraflash_dataset_weighted(dataset = SampleFirst, export = True)
    



################                                         ################
################     Weighted thickness calulation d     ################
################                                         ################



base_path = SampleFirst['outputpath'] + os.sep


file_paths = [f"{base_path}MgO_RT_{i}_optimization_result.txt" for i in range(0, number_measurements)]

weighted_average, weighted_average_error, statistic_average, statistic_average_error  = TDS.calculate_weighted_thickness(file_paths = file_paths, f_min = SampleFirst['average_fmin'], f_max =  SampleFirst['average_fmax'], number_measurements  = number_measurements)            


TDS.export_average_nkd_optimazation_result(file_paths = file_paths, base_path = base_path)




################                                         ################
################       Fixed solving of n and k          ################
################                                         ################



for i in range(number_measurements):      
    Sample = {'name': 'MgO_RT_%i' % (i),
          'file_reference': os.path.join(test_data_dir, f'File2000avg_22072024_s0_Air_L{i}.csv'),
          'file_sample': os.path.join(test_data_dir, f'File2000avg_22072024_s1_MgO_L{i}.csv'),
          'file_background': SampleFirst['file_background'],
          'outputpath': SampleFirst['outputpath'],
          'subtract_background': SampleFirst['subtract_background'],
          'd_estimate': SampleFirst['d_estimate'],
          'nmin': SampleFirst['nmin'],
          'nmax': SampleFirst['nmax'],
          'n_estimate': SampleFirst['n_estimate'],
          'kmin': SampleFirst['kmin'],
          'kmax':SampleFirst['kmax'],
          'T_window': SampleFirst['T_window'],
          'window_t_offset': SampleFirst['window_t_offset'],
          'window_type': SampleFirst['window_type'],#'Kaiser-Bessel',#'rect'
          'window_parameter': SampleFirst['window_parameter'],
          'fmin': SampleFirst['fmin'],
          'fmax': SampleFirst['fmax'],
          'model': SampleFirst['model'],
          'model_parameters': SampleFirst['model_parameters'],
          'interpolate': SampleFirst['interpolate'],
          'interpolation_method': SampleFirst['interpolation_method'],
          'interpolation_dt_ratio': SampleFirst['interpolation_dt_ratio'],
          'thickness_averaging': SampleFirst['thickness_averaging'], #'fixed_interval', #'weighted', #'threshold'
          'average_fmin': SampleFirst['average_fmin'],
          'average_fmax': SampleFirst['average_fmax'],
          'average_threshold': SampleFirst['average_threshold']}
    print(Sample['name'])
    
    Sample_res = TDS.evaluate_Teraflash_dataset_weighted_fixed_thickness(dataset = Sample, export = True, weighted = weighted_average, weighted_error = weighted_average_error  )
    

################                                   ################
################     Plot resulting n, k and d     ################
################                                   ################


f_sample, mean_sample, mean_fixed_thickness_sample = TDS.read_and_export_TDS_data_weighted(basepath = Sample['outputpath'], 
                                                                      material = 'MgO', 
                                                                      export = True, 
                                                                      header = '# Sample MgO, 0.500 mm MgO, supergaussian window, weighted values of fixed thickness optimization, no background subtraction, new detector', 
                                                                      measurements = list(range(number_measurements)),
                                                                      weighted = weighted_average,
                                                                      weighted_error = weighted_average_error )
                                                             






TDS.plot_nkd_weighted2(f = f_sample, 
             mean = mean_sample, 
             mean_fixed_thickness = mean_fixed_thickness_sample, 
             statistic_average = statistic_average,
             statistic_average_error = statistic_average_error,
             flim = [0.1, 4.5], 
             nlim = [2.5, 4.0], 
             klim = [-0.001, 0.02], 
             dlim = [450, 600], 
             f_interval = [SampleFirst['average_fmin'], SampleFirst['average_fmax']], 
             title = r'0.5mm MgO, new detector Solver Version 3', 
             export = True, 
             path = base_path)








